@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010, 2012 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@ from __future__ import with_statement
 import os
 from cStringIO import StringIO
 
-import Image as PIL
+from PIL import Image
 
 import djvu.decode
 
@@ -56,8 +56,8 @@ def _test_from_file(base_filename, format):
     else:
         # The result might be still correct, even if the strings are different.
         # Think of BMP format and its padding bytes.
-        expected = PIL.open(expected_filename)
-        result = PIL.open(StringIO(result))
+        expected = Image.open(expected_filename)
+        result = Image.open(StringIO(result))
         assert_equal(result.format, expected.format)
         assert_equal(result.size, expected.size)
         assert_equal(result.mode, expected.mode)
